#include "zcommon.acs"
#library "lelweps"

#include "commonFuncs.h"

#include "const.h"
#include "funcs.h"

#include "cvars.h"
#include "lul.h"
#include "blackhole.h"
#include "powerful.h"
#include "launchers.h"
#include "healpistol.h"
#include "beams.h"
#include "geo.h"

script 420 (void) net
{
    if (GetCVar("lelweps_giveallweapons") < 1)
    {
        Print(s:"You are not allowed to get the Lelweps.");
        terminate;
    }

    GiveLelwep(-1);
}

script 452 ENTER
{
    ACS_ExecuteWithResult(429, 0);
}

script 453 RESPAWN
{
    ACS_ExecuteWithResult(429, 1);
}

script 454 (int scaleI, int scaleF, int speedF)
{
    int scale, x, y, z, mag;
    x = GetActorVelX(0); y = GetActorVelY(0); z = GetActorVelZ(0);

    if (speedF == 0) { scale = itof(scaleI) + (scaleF * 0.01); }
    else
    {
        mag = magnitudeThree_f(x, y, z);

        if (mag == 0)
        {
            x = random(-10.0, 10.0);
            y = random(-10.0, 10.0);
            z = random(-10.0, 10.0);
            mag = magnitudeThree_f(x, y, z);
        }

        scale = FixedDiv(speedF * 1.0, mag);
    }

    SetActorVelocity(0, FixedMul(x, scale), FixedMul(y, scale), FixedMul(z, scale), 0, 0);
    // PrintBold(s:"(", f:x, s:", ", f:y, s:", ", f:z, s:"): ", f:scale, s:" -> (", f:GetActorVelX(0), s:", ", f:GetActorVelY(0), s:", ", f:GetActorVelZ(0), s:")");
    // printf("(%f, %f, %f): %f -> (%f, %f, %f)\n", x, y, z, scale, GetActorVelX(0), GetActorVelY(0), GetActorVelZ(0));
}

script 458 (int count, int high, int low)
{
    int ret = 0;
    int i;
    if (!low) { low = -high; }

    for (i = 0; i < count; i++) { ret += random(low, high); }
    SetResultValue(ret);
}

script 429 (int respawning)
{
    int i;

    if (GetCVar("lelweps_giveallweapons") >= 2)
    {
        GiveLelwep(-1);
    }
    else if (GetCVar("lelweps_giveallweapons") == 1)
    {
        Print(s:"You can get the Lelweps at any time (script 420).");
    }

    for (i = 0; i < LELWEPCOUNT; i++)
    {
        if (startWithLelwep(i)) { GiveLelwep(i); }
    }

    if (!respawning)
    {
        while (1)
        {
            if (keyDown(BT_ATTACK)) { GiveInventory("LeftFiring", 1); }
            else { TakeInventory("LeftFiring", 1); }

            if (keyDown(BT_ALTATTACK)) { GiveInventory("RightFiring", 1); }
            else { TakeInventory("RightFiring", 1); }

            for (i = 0; i < LELWEPCOUNT; i++)
            {
                if (lelwepsCurStart[i] && !lelwepsPrevStart[i]) { ACS_ExecuteWithResult(472, i, 1); }
                if (!lelwepsCurStart[i] && lelwepsPrevStart[i]) { ACS_ExecuteWithResult(472, i, 0); }
            }

            Delay(1);
        }

    }
}

script 461 (void)
{
    int velx, vely, velz;
    int ovelx, ovely, ovelz;
    int mag, omag;
    int launchdir, launchzdir, launchtime;
    int nx, ny, nz;

    launchdir = random(0, 1.0);
    launchzdir = random(-0.05, 0.25);
    launchtime = 3;

    while (1)
    {
        ovelx = velx;
        ovely = vely;
        ovelz = velz;
        omag  = mag;
        velx = GetActorVelX(0);
        vely = GetActorVelY(0);
        velz = GetActorVelZ(0);
        mag  = magnitudeThree_f(velx, velz, velz);

        if ((mag + 4.0) < omag || mag < 4.0)
        {
            launchdir += random(0.25, 0.75);
            launchdir %= 1.0;
            launchzdir = random(-0.05, 0.25);
            launchtime = 3;
        }

        if (launchtime > 0)
        {
            nx = FixedMul(15 * cos(launchdir), cos(launchzdir));
            ny = FixedMul(15 * sin(launchdir), cos(launchzdir));
            nz = 15 * sin(launchzdir);
            SetActorVelocity(0, nx,ny,nz, 1,0);
        }

        launchtime = max(launchtime-1, 0);
        Delay(1);
    }
}

script 472 (int index, int mode)
{
    int wep = Lelweps[index][0];
    int pln = PlayerNumber();
    int i, slot = -1;
    
    if (mode) { GiveLelwep(index); }
    else { TakeLelwep(index); }

    for (i = 0; i < LELWEPCOUNT; i++)
    {
        if (lelwepsNotifyOffsets[pln][i] == index || lelwepsNotifyOffsets[pln][i] == 0)
        {
            slot = i;
            break;
        }
    }

    if (slot == -1) { terminate; }

    SetHudSize(800, 600, 1);
    
    HudMessage(s:"The ", s:wep, s:" has been ", s:cond(mode, "\cdENABLED", "\cgDISABLED"), s:"\c-.";
                HUDMSG_FADEOUT, 19102+slot, CR_WHITE,
                600.4, 75.0 + (16.0 * slot), 2.0, 0.5);
    
    lelwepsNotifyOffsets[pln][slot] = index;
    Delay(70);
    lelwepsNotifyOffsets[pln][slot] = 0;
}

script 473 (int which)
{
    int ret;

    switch (which)
    {
      case 0:
        ret = (ConsolePlayerNumber() != -1);
        break;

      case 1:
        SetActorPosition(0, GetActorX(0), GetActorY(0), GetActorFloorZ(0), 0);
        ret = ftoi(GetActorFloorZ(0));
        break;
    }

    SetResultValue(ret);
}


script 481 (int zdrop, int time, int zmax)
{
    if (!zmax) { zmax = 5; }

    int zoff = min(0, GetActorVelZ(0));
    int i;

    for (i = 0; i < time; i++)
    {
        SetActorVelocity(0, GetActorVelX(0), GetActorVelY(0), max((-itof(zdrop)/zmax)+zoff, GetActorVelZ(0)-(itof(zdrop)/100)), 0,0);
        SetActorProperty(0, APROP_Alpha, min(1.0, 1.0 + itof((2*time) - (3*i)) / time) );
        Delay(1);
    }

    if (PlayerNumber() == -1) { Thing_Remove(0); }
}



#define CLOCKWISE           0
#define COUNTERCLOCKWISE    1

script 496 (void)
{
    int myTID = defaultTID(-1);
    SetActivatorToTarget(0);
    int myOwner = defaultTID(-1);
    int ownerPln = PlayerNumber();

    int fireAngle = GetActorAngle(0);
    int firePitch = GetActorPitch(0);

    firePitch = min(-0.1, firePitch);

    int haloAngle = 0;
    int time = 0;
    int freetid, i, j;
    int vx, vy, vz, vmult;
    int myx, myy, myz;

    SetActivator(myTID);

    while (1)
    {
        if (CheckActorInventory(myOwner, "EndJumpPads") || !PlayerInGame(ownerPln)) { break; }
        if (!CheckInventory("JumpPadLaunching")) { Delay(1); continue; }

        myx = GetActorX(0); myy = GetActorY(0); myz = GetActorZ(0);

        Spawn("JumpPadMarker",  GetActorX(0) + (32 * cos(haloAngle)),
                                GetActorY(0) + (32 * sin(haloAngle)),
                                GetActorZ(0));

        if (time % 18 == 9)
        {
            freetid = unusedTID(15000, 25000);

            for (i = 0; i < 18; i++)
            {
                j = 0.0555555 * i;
                Spawn("JumpPadMarker2", GetActorX(0) + (28 * cos(j)),
                                        GetActorY(0) + (28 * sin(j)),
                                        GetActorZ(0),
                                        freetid);

                
                vx = 2 * FixedMul(cos(fireAngle), cos(firePitch));
                vy = 2 * FixedMul(sin(fireAngle), cos(firePitch));
                vz = 2 * -sin(firePitch);
                
                vmult = abs(j - fireAngle);
                while (vmult > 0.5) { vmult = abs(1.0 - vmult); };

                vmult = FixedMul(vmult, 8 * abs(-0.25 - firePitch));
                vmult += 2.0;

                vx = FixedMul(vx, vmult);
                vy = FixedMul(vy, vmult);
                vz = FixedMul(vz, vmult);

                SetActorVelocity(freetid, vx, vy, vz, 0, 0);

                Thing_ChangeTID(freetid, 0);
            }
        }

        if (SetActivatorToTarget(0))
        {
            if (!CheckInventory("NoJump"))
            {
                SetActorPosition(0, myx, myy, myz, 0);

                SetActorVelocity(0, 30 * FixedMul(cos(fireAngle), cos(firePitch)),
                                    40 * FixedMul(sin(fireAngle), cos(firePitch)),
                                    40 * -sin(firePitch), 0, 0);

                i = ftoi(abs(fireAngle - GetActorAngle(0)) * 360);
                if (i > 180) { i = 360 - i; }

                //Log(f:fireAngle, s:" - ", f:GetActorAngle(0), s:": ", d:i);
                
                if (GetActorAngle(0) < fireAngle) { j = GetActorAngle(0) + 1.0; }
                else { j = GetActorAngle(0); }

                ACS_ExecuteWithResult(497);
                ACS_ExecuteWithResult(498, i, 3, cond(j - fireAngle <= 0.5, CLOCKWISE, COUNTERCLOCKWISE));

                ActivatorSound("lelweps/jumppad", 127);
            }
        }

        SetActivator(myTID);
            
        haloAngle += 0.015;

        time++;
        Delay(1);
    }

    Thing_Remove(myTID);
}

script 497 (void)
{
    int time = 0;
    int groundstat = 0;
    GiveInventory("NoJump", 1);

    while (1)
    {
        if (GetActorZ(0) != GetActorFloorZ(0) && groundstat == 0) { groundstat = 1; }
        if (GetActorZ(0) == GetActorFloorZ(0) && groundstat == 1) { groundstat = 2; SetActorVelocity(0, 0,0,0, 0, 0); }
        if (time == 4) { TakeInventory("NoJump", 1); }

        if (time >= 4 && groundstat == 2) { break; }

        time++;
        Delay(1);
    }
}

script 498 (int degrees, int factor, int direction) net clientside
{
    if (degrees < 0)
    {
        degrees *= -1;
        direction = cond(direction == CLOCKWISE, COUNTERCLOCKWISE, CLOCKWISE);
    }
    
    factor = cond(factor, factor, 4);

    int prevDegrees, addDegrees, curAngle;
    int curDegrees = 0;
    int floatDegrees = itof(degrees);
    int dirMult = cond(direction == CLOCKWISE, -1, 1);

    while (curDegrees < (floatDegrees - 0.1))
    {
        prevDegrees = curDegrees;
        addDegrees = (floatDegrees - curDegrees) / factor;
        curDegrees += addDegrees;

        SetActorAngle(0, GetActorAngle(0) + ((addDegrees * dirMult) / 360));
        Delay(1);
    }

    addDegrees = floatDegrees - curDegrees;
    SetActorAngle(0, GetActorAngle(0) + ((addDegrees * dirMult) / 360));
}
